package mffs;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;

import mffs.projectors.TileProjector;
import mffs.projectors.TileProjectorArea;
import mffs.projectors.TileProjectorDeflector;
import mffs.projectors.TileProjectorDirectional;
import mffs.projectors.TileProjectorExtender;
import mffs.projectors.TileProjectorReactor;
import mffs.projectors.TileProjectorTube;
import net.minecraft.src.forge.ISpecialResistance;
import net.minecraft.src.forge.ITextureProvider;
import net.minecraft.src.*;

public class BlockMachine extends BlockContainer implements ITextureProvider, ISpecialResistance {

	public static final int META_CORE = 0;
	public static final int META_PROJ_AREA = 1;
	public static final int META_PROJ_DIR = 2;
	public static final int META_PROJ_DEFLECTOR = 3;
	public static final int META_PROJ_TUBE = 4;
	public static final int META_PROJ_EXTENDER = 5;
	public static final int META_EU_INJECTOR = 6;
	public static final int META_PROJ_REACTOR = 7;
	public static final int META_REACTOR_COOLER = 8;
	public static final int META_REACTOR_MONITOR = 9;
	
	public static final int MAX_META = 10;
	
	private int blockid;

	public BlockMachine(int i) {

		super(i, Material.iron);
		blockid = i;
		setHardness(3F);
		setResistance(50F);
		setStepSound(soundMetalFootstep);

	}

	public int getBlockid() {
		return blockid;
	}

	public int getGui(World world, int i, int j, int k, EntityPlayer entityplayer) {
		switch (world.getBlockMetadata(i, j, k)) {
		case 0:
			return mod_ModularForceFieldSystem.GUI_ID_GENERATOR;
		case 1:
			return mod_ModularForceFieldSystem.GUI_AREA_PROJECTOR;
		case 2:
			return mod_ModularForceFieldSystem.GUI_DIRECTIONAL_PROJECTOR;
		case 3:
			return mod_ModularForceFieldSystem.GUI_DEFLECTOR_PROJECTOR;
		case 4:
			return mod_ModularForceFieldSystem.GUI_TUBE_PROJECTOR;
		case 5:
			return mod_ModularForceFieldSystem.GUI_DIRECTIONAL_UPGRADE;
		case 7:
			return mod_ModularForceFieldSystem.GUI_REACTOR_FIELD;
		case 8:
			return mod_ModularForceFieldSystem.GUI_REACTOR_COOLER;
		case 9:
			return mod_ModularForceFieldSystem.GUI_REACTOR_MONITOR;
		default:
			return -1;
		}
	}

	public boolean blockActivated(World world, int i, int j, int k, EntityPlayer entityplayer) {

		if (entityplayer.getCurrentEquippedItem() != null && entityplayer.getCurrentEquippedItem().itemID == Block.lever.blockID) {
			return false;
		}
		if (entityplayer.getCurrentEquippedItem() != null && entityplayer.getCurrentEquippedItem().itemID == mod_ModularForceFieldSystem.MFFSUpgrades.blockID) {
			return false;
		}
		if (entityplayer.getCurrentEquippedItem() != null && entityplayer.getCurrentEquippedItem().itemID == mod_ModularForceFieldSystem.MFFSMaschines.blockID) {
			return false;
		}

		if (entityplayer.getCurrentEquippedItem() != null && (entityplayer.getCurrentEquippedItem().getItem() instanceof ItemMFD)) {
			return false;
		}
		
		if(world.isRemote)
			return true;

		int gui = getGui(world, i, j, k, entityplayer);

		if(gui < 0)
			return false;

		entityplayer.openGui(mod_ModularForceFieldSystem.instance, gui, world, i, j, k);
		return true;
	}

	public TileEntityMaschines getBlockEntity() {
		return null;
	}

	public void onBlockAdded(World world, int i, int j, int k) {
		if(world.isRemote) {
			return;
		}
		int meta = world.getBlockMetadata(i, j, k);
		if (meta == 0) {
			TileEntityGeneratorCore tileentityblock = (TileEntityGeneratorCore) world.getBlockTileEntity(i, j, k);
			tileentityblock.addtogrid();
		}
		if (meta == 1) {
			TileProjectorArea tileentityblock = (TileProjectorArea) world.getBlockTileEntity(i, j, k);
			tileentityblock.addtogrid();
		}
		if (meta == 2) {
			TileProjectorDirectional tileentityblock = (TileProjectorDirectional) world.getBlockTileEntity(i, j, k);
			tileentityblock.addtogrid();
		}
		if (meta == 3) {
			TileProjectorDeflector tileentityblock = (TileProjectorDeflector) world.getBlockTileEntity(i, j, k);
			tileentityblock.addtogrid();
		}
		if (meta == 4) {
			TileProjectorTube tileentityblock = (TileProjectorTube) world.getBlockTileEntity(i, j, k);
			tileentityblock.addtogrid();
		}
		if (meta == 7) {
			TileProjectorReactor tileentityblock = (TileProjectorReactor) world.getBlockTileEntity(i, j, k);
			tileentityblock.addtogrid();
		}

	}

	public void onBlockRemoval(World world, int i, int j, int k) {
		if(world.isRemote) {
			return;
		}
		
		TileEntity te = world.getBlockTileEntity(i, j, k);
		
		if(te instanceof TileEntityGeneratorCore) {
			((TileEntityGeneratorCore)te).removefromgrid();
		} else if(te instanceof TileProjector) {
			((TileProjector)te).destroyField();
			((TileProjector)te).removefromgrid();
		} else if(te instanceof TileEntityReaktorMonitor) {
			((TileEntityReaktorMonitor)te).removefromgrid();
		}

		if (te instanceof TileProjector) {
			ItemStack itemstack = ((TileProjector) te).getStackInSlot(0);
			if (itemstack != null) {
				world.spawnEntityInWorld(new EntityItem(world, i + 0.5f, j + 0.5f, k + 0.5f, itemstack));
			}
		}

		world.removeBlockTileEntity(i, j, k);

	}

	public void onBlockPlacedBy(World world, int i, int j, int k, EntityLiving entityliving) {
		if(world.isRemote) {
			return;
		}

		TileEntityMaschines tileentityblock = (TileEntityMaschines) world.getBlockTileEntity(i, j, k);

		int l = MathHelper.floor_double((double) ((entityliving.rotationYaw * 4F) / 360F) + 0.5D) & 3;
		int i1 = Math.round(entityliving.rotationPitch);
		if (i1 >= 65) {
			tileentityblock.setFacing((short) 1);
		} else if (i1 <= -65) {
			tileentityblock.setFacing((short) 0);
		} else if (l == 0) {
			tileentityblock.setFacing((short) 2);
		} else if (l == 1) {
			tileentityblock.setFacing((short) 5);
		} else if (l == 2) {
			tileentityblock.setFacing((short) 3);
		} else if (l == 3) {
			tileentityblock.setFacing((short) 4);
		}
	}

	protected int damageDropped(int i) {
		return i;

	}

	public TileEntityMaschines getBlockEntity(int i) {
		switch (i) {
		case 0:
			return new TileEntityGeneratorCore();
		case 1:
			return new TileProjectorArea();
		case 2:
			return new TileProjectorDirectional();
		case 3:
			return new TileProjectorDeflector();
		case 4:
			return new TileProjectorTube();
		case 5:
			return new TileProjectorExtender();
		case 6:
			return new TileEntityGeneratorEUInjector();
		case 7:
			return new TileProjectorReactor();
		case 8:
			return new TileEntityReaktorCooler();
		case 9:
			return new TileEntityReaktorMonitor();
		}
		return null;
	}

	public String getTextureFile() {

		return "/mffs_grafik/machines.png";
	}

	public int idDropped(int i, Random random) {
		switch (i) {
		default:
			return blockID;
		}
	}

	public static boolean isActive(IBlockAccess iblockaccess, int i, int j, int k) {
		TileEntity tileentity = iblockaccess.getBlockTileEntity(i, j, k);
		if (tileentity instanceof TileEntityMaschines) {
			return ((TileEntityMaschines) tileentity).getActive();
		} else {
			return false;
		}
	}

	public int getBlockTexture(IBlockAccess iblockaccess, int i, int j, int k, int l) {
		TileEntity tileentity = iblockaccess.getBlockTileEntity(i, j, k);
		short facing = (tileentity instanceof TileEntityMaschines) ? ((TileEntityMaschines) tileentity).getFacing() : 0;
		int meta = iblockaccess.getBlockMetadata(i, j, k);

		if (isActive(iblockaccess, i, j, k)) {
			if (facing == l) {
				return (meta * 16) + 7;
			}
			if (facing != 0 && l == 0) {
				return (meta * 16) + 7 + 1;
			}
			return (meta * 16) + 7 + l;

		} else {
			if (facing == l) {
				return (meta * 16);
			}
			if (facing != 0 && l == 0) {
				return (meta * 16) + 1;
			}
			return (meta * 16) + l;
		}
	}

	public int getBlockTextureFromSideAndMetadata(int i, int j) {

		return (j * 16) + i - 1;
	}

	public float getSpecialExplosionResistance(World world, int i, int j, int k, double d, double d1, double d2, Entity entity) {

		if (world.getBlockTileEntity(i, j, k) instanceof TileEntityMaschines) {
			TileEntity tileentity = world.getBlockTileEntity(i, j, k);
			if (((TileEntityMaschines) tileentity).getActive()) {
				return 60000F;
			} else {
				return 50F;
			}
		}
		return 50F;
	}
	
	@Override
	public void addCreativeItems(ArrayList l) {
		for(int k = 0; k < MAX_META; k++)
			l.add(new ItemStack(this, 1, k));
	}

}
